#include "astodbc/astodbc.h"

/*
 * to use odbc-vm-routines, set USE_ODBC_VM_INTERFACE to 1
 * in asterisk/apps/Makefile , then put this file into
 * asterisk-addson/ and (re)build asterisk.
*/
void*   odbc_conn;
char	dbconn_string[512];

#ifdef USESQLVM
static struct ast_vm_user *find_user(struct ast_vm_user *ivm, char *context, char *mailbox)
{
	void* odbc_query;
	const char* odbc_result;

	char query[1024];
	char options[512] = "";
	struct ast_vm_user *retval;

	retval=malloc(sizeof(struct ast_vm_user));
#if _DEBUG
	fprintf(stderr,"vm_find_user: mailbox=%s,context=%s\n",mailbox,context ? context : "default" );
#endif
	if (retval) {
		memset(retval, 0, sizeof(struct ast_vm_user));
		retval->alloced=1;
		if (mailbox) {
			strcpy(retval->mailbox, mailbox);
		}
		if (context) {
			strcpy(retval->context, context);
		}
		else {
			strcpy(retval->context, "default");
		}
		populate_defaults(retval);
		sprintf(query,db_getvmuser_query,mailbox,context ? context : "default" );
#if _DEBUG
		fprintf(stderr,"vm_find_user: query = %s\n",query);
#endif
		odbc_result = ast_odbc_query_new(odbc_conn,query,&odbc_query);
		if (strlen(odbc_result)==0) {
			if( ast_odbc_query_next(odbc_result) )  {
				strncpy(retval->password,ast_odbc_query_getstring(odbc_query,"password",""),sizeof(revtal->password)-1);
				strncpy(retval->fullname,ast_odbc_query_getstring(odbc_query,"fullname",""),sizeof(revtal->fullname)-1);
				strncpy(retval->email   ,ast_odbc_query_getstring(odbc_query,"email"   ,""),sizeof(revtal->email)   -1);
				strncpy(retval->pager   ,ast_odbc_query_getstring(odbc_query,"pager"   ,""),sizeof(revtal->pager)   -1);
				strncpy(options         ,ast_odbc_query_getstring(odbc_query,"options" ,""),sizeof(options) -1);
				if( strlen(retval->options) ) {
					apply_options(retval,options);
				}
				odbc_query = ast_odbc_query_delete(odbc_query);
				return retval;
			}
			else {
				ast_log(LOG_WARNING,"ast_odbc_query_new did not find a mailbox for %s\n",mailbox);
			}
		}
		else {
			ast_log(LOG_WARNING,"ast_odbc_query_new: query error (%s)\n",odbc_result);
		}
		free(retval);
	}
	else {
		ast_log(LOG_WARNING,"unable to allocate %d bytes for ast_vm_user\n",sizeof(ast_vm_user));
	}
	return NULL;
}

static void vm_change_password(struct ast_vm_user *vmu, char *password)
{
	char query[1024];
	const char* odbc_result;
	void* odbc_query;
	sprintf(query,db_changepassword_query,password,vmu->password,vmu->mailbox,vmu->context ? vmu->context : "default" );
#if _DEBUG
	fprintf(stderr,"vm_change_password: query = %s\n",query);
#endif
	odbc_result = ast_odbc_query_new(odbc_conn,query,&odbc_query);
	if (strlen(odbc_result)!=0) {
		ast_log(LOG_WARNING, "Error executing query %s: %s",query,odbc_result);
	}
	else {
		odbc_query = ast_odbc_query_delete(odbc_query);
		strcpy(vmu->password, password);
	}
}

static void reset_user_pw(char *context, char *mailbox, char *password)
{
	char query[1024];
	const char* odbc_result;
	void* odbc_query;
	sprintf(query,db_setpassword_query,password,mailbox,context ? context : "default" );
#if _DEBUG
	fprintf(stderr,"reset_user_pw: query = %s\n",query);
#endif
	odbc_result = ast_odbc_query_new(odbc_conn,query,&odbc_query);
	if (strlen(odbc_result)!=0) {
		ast_log(LOG_WARNING, "Error executing query %s: %s",query,odbc_result);
	}
	else {
		odbc_query = ast_odbc_query_delete(odbc_query);
	}
}
#endif /* USESQLVM */

static int sql_init(void)
{
	const char* odbc_result;
	ast_verbose( VERBOSE_PREFIX_3 "Logging into ODBC database: %s\n", dbconn_string);
	odbc_result = ast_odbc_conn_new(dbconn_string,1,0,&odbc_conn);
	if (strlen(odbc_result)!=0) {
		ast_log(LOG_WARNING, "Error Logging into database %s: %s\n",dbconn_string,odbc_result);
		return(-1);
	}
	return(0);
}

static void sql_close(void)
{
	odbc_conn = ast_odbc_conn_delete(odbc_conn);
}

